--[[
Interface: 1.5.1.0 b1580

Author: GtX (LS-Modcompany)

Contributors:
Internal Cover operation based on Original cover.lua by Giants Software.

History:
V 1.0 @ 03.01.2018

Support only at: http://ls-modcompany.com

Designed to be used on Animal Trailers. If used in Auto mode it forces the animation open when animals are loaded and auto closes all when trailer is empty.
The inputButton is disabled when animals are loaded so not to crush them.

'doorAnimation' Simple animation that can be operated inside or outside depending on your settings.

'coolingFans' Just like normal RotationNodes different settings can be set. These nodes will turn on when the trailer has animals loaded.

EXAMPLE FOR XML:

<livestockTrailerExtension>
	<externalTrigger index="0>0|12" distance="2" />
	<doorAnimation name="toggleDoors" openText="action_openRearDoors" closeText="action_closeRearDoors" internalOperation="false" externalOperation="true"/>	
	<coverAnimation name="toggleCover" openOnBuy="false" autoOpenClose="true" openText="action_openAnimalCover" closeText="action_closeAnimalCover" internalOperation="true" externalOperation="false"/>

	<fanRotationNodes>
		<fanRotationNode index="0>1" type="coolingFans" rotSpeed="-900"  rotAxis="1" turnOnFadeTime="2" turnOffFadeTime="4" />
	</fanRotationNodes>	
</livestockTrailerExtension/>
]]

LivestockTrailerExtension = {}

function LivestockTrailerExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations)
end

function LivestockTrailerExtension:preLoad(savegame)
    self.getIsToggleCoverAllowed = LivestockTrailerExtension.getIsToggleCoverAllowed
    self.setCoverState = LivestockTrailerExtension.setCoverState
	self.setDoorState = LivestockTrailerExtension.setDoorState
	self.animalsLoaded = LivestockTrailerExtension.animalsLoaded
	self.isUsedOn = LivestockTrailerExtension.isUsedOn
	self.isInTriggerNode = LivestockTrailerExtension.isInTriggerNode
end

function LivestockTrailerExtension:load(savegame)
	local key = "vehicle.livestockTrailerExtension"	
	
	self.nodeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key..".externalTrigger#index"))	
	if self.nodeIndex ~= nil then
		self.triggerDistance = Utils.getNoNil(getXMLFloat(self.xmlFile, key..".externalTrigger#distance"), 1.0)
	end
	
	self.doorsAnimation = getXMLString(self.xmlFile, key..".doorAnimation#name")
	if self.doorsAnimation ~= nil then
		self.doorsOpenText = Utils.getNoNil(getXMLString(self.xmlFile, key..".doorAnimation#openText"), "action_openRearDoors")
		self.doorsCloseText = Utils.getNoNil(getXMLString(self.xmlFile, key..".doorAnimation#closeText"), "action_closeRearDoors")
		self.doorsInternal = Utils.getNoNil(getXMLBool(self.xmlFile, key..".doorAnimation#internalOperation"), true)
		self.doorsExternal = Utils.getNoNil(getXMLBool(self.xmlFile, key..".doorAnimation#externalOperation"), false)
		
		local doorsOpen = false		
		if savegame ~= nil then
            doorsOpen = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#doorsOpen"), doorsOpen)
        end

		self:setDoorState(doorsOpen, true)
	else
        self.doorsOpen = false
	end
	
    self.coverAnimation = getXMLString(self.xmlFile, key..".coverAnimation#name")
    if self.coverAnimation ~= nil then
		self.coverOpenText = Utils.getNoNil(getXMLString(self.xmlFile, key..".coverAnimation#openText"), "action_openAnimalCover")
		self.coverCloseText = Utils.getNoNil(getXMLString(self.xmlFile, key..".coverAnimation#closeText"), "action_closeAnimalCover")
		self.coverInternal = Utils.getNoNil(getXMLBool(self.xmlFile, key..".coverAnimation#internalOperation"), true)
		self.coverExternal = Utils.getNoNil(getXMLBool(self.xmlFile, key..".coverAnimation#externalOperation"), false)
	
        local isOpen = Utils.getNoNil(getXMLBool(self.xmlFile, key..".coverAnimation#openOnBuy"), false)
		
        if savegame ~= nil then
            isOpen = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#isCoverOpen"), isOpen)
        end
		
		self.forceOpen = isOpen
        self:setCoverState(isOpen, true)
		self.autoOpenClose = Utils.getNoNil(getXMLBool(self.xmlFile, key..".coverAnimation#autoOpenClose"), true)
		self.canChangeCover = true
    else
        self.isCoverOpen = true
    end	
	
	if self.isClient then
	    self.fanRotationNodes = Utils.loadRotationNodes(self.xmlFile, {}, key..".fanRotationNodes.fanRotationNode", "coolingFans", self.components)
	end
	
end

function LivestockTrailerExtension:getSaveAttributesAndNodes(nodeIdent)
    local attributes = ""
	local nodes = nil
	
	if self.coverAnimation ~= nil then
        attributes = attributes.. 'isCoverOpen="'..tostring(self.isCoverOpen)..'" '
    end
	
	if self.doorsAnimation ~= nil then
        attributes = attributes.. 'doorsOpen="'..tostring(self.doorsOpen)..'"'
    end
	
    return attributes, nodes
end

function LivestockTrailerExtension:readStream(streamId, connection)
    if connection:getIsServer() then
        if self.coverAnimation ~= nil then
            local isOpen = streamReadBool(streamId)
            self:setCoverState(isOpen, true)
        end
		
		if self.doorsAnimation ~= nil then
			local doorsOpen = streamReadBool(streamId)
			self:setDoorState(doorsOpen, true)
		end
    end
end

function LivestockTrailerExtension:writeStream(streamId, connection)
    if not connection:getIsServer() then
        if self.coverAnimation ~= nil then
            streamWriteBool(streamId, self.isCoverOpen)
        end
		
		if self.doorsAnimation ~= nil then
			streamWriteBool(streamId, self.doorsOpen)
		end
    end
end

function LivestockTrailerExtension:update(dt)
	if self.coverAnimation ~= nil then
		if self.autoOpenClose then
			if self:animalsLoaded() or self.forceOpen then
				if not self.isCoverOpen then
					self:setCoverState(true)
				end
				if self:animalsLoaded() then
					self.canChangeCover = false
					self.forceOpen = false
				end
			else
				if self.isCoverOpen then
					self:setCoverState(false)
				end
				if not self:animalsLoaded() then
					self.canChangeCover = true
				end
			end
		end
		
		if self:getIsActiveForInput() then
			if self.canChangeCover and self.coverInternal then
				if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
					if self:getIsToggleCoverAllowed(not self.isCoverOpen) then
						self.forceOpen = not self.forceOpen
						-- self:setCoverState(not self.isCoverOpen)
					end
				end
			end
		end
	end
	
	if self.doorsAnimation ~= nil then
		if self:getIsActiveForInput() then
			if self.doorsInternal then
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
					self:setDoorState(not self.doorsOpen)
				end
			end
		end
	end
		
	if self.nodeIndex ~= nil then
		if self:isInTriggerNode() then	
			if self.coverAnimation ~= nil and self.coverExternal then
				if self.canChangeCover then
					if self.isCoverOpen then
						g_currentMission:addHelpButtonText(g_i18n:getText(self.coverCloseText), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL)
					else                                                                                                  
						g_currentMission:addHelpButtonText(g_i18n:getText(self.coverOpenText), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL)
					end
					
					if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
						if self:getIsToggleCoverAllowed(not self.isCoverOpen) then	
							self.forceOpen = not self.forceOpen
							-- self:setCoverState(not self.isCoverOpen)
						end
					end
				else
					g_currentMission:addExtraPrintText(g_i18n:getText("warning_coverLocked"))
				end
			end
			
			if self.doorsAnimation ~= nil then
				if self.doorsExternal then
					if self.doorsOpen then
						g_currentMission:addHelpButtonText(g_i18n:getText(self.doorsCloseText), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_NORMAL)
					else                                                                                                  
						g_currentMission:addHelpButtonText(g_i18n:getText(self.doorsOpenText), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_NORMAL)
					end
					
					if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
						if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
							self:setDoorState(not self.doorsOpen)
						end
					end
				end
			end
		end
	end
	
	if self.isClient then
		local runFans = self:animalsLoaded() and self:isUsedOn()
		Utils.updateRotationNodes(self, self.fanRotationNodes, dt, runFans)
	end

end

function LivestockTrailerExtension:draw()
	if self.coverInternal then
		if self.canChangeCover then
			if self.isClient and self:getIsActiveForInput(true) then
				if self.coverAnimation ~= nil and self:getIsToggleCoverAllowed(not self.isCoverOpen) then
					if self.isCoverOpen then
						g_currentMission:addHelpButtonText(g_i18n:getText(self.coverCloseText), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL)
					else                                                                                                  
						g_currentMission:addHelpButtonText(g_i18n:getText(self.coverOpenText), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL)
					end
				end
			end
		else
			g_currentMission:addExtraPrintText(g_i18n:getText("warning_coverLocked"))
		end
    end
	
	if self.doorsInternal then
		if self.isClient and self:getIsActiveForInput(true) then
			if self.doorsAnimation ~= nil then
				if self.doorsOpen then
					g_currentMission:addHelpButtonText(g_i18n:getText(self.doorsCloseText), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_NORMAL)
				else                                                                                                  
					g_currentMission:addHelpButtonText(g_i18n:getText(self.doorsOpenText), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_NORMAL)
				end
            end
        end
    end	
end

function LivestockTrailerExtension:getIsToggleCoverAllowed(isCoverOpen)
    if self.tipState ~= nil then
        return self.tipState == Trailer.TIPSTATE_CLOSED
    end
    return true
end

function LivestockTrailerExtension:setCoverState(isOpen, noEventSend)
    if self.coverAnimation ~= nil then
        SetCoverStateEvent.sendEvent(self, isOpen, noEventSend)
        self.isCoverOpen = isOpen
        if isOpen then
            self:playAnimation(self.coverAnimation, -1, self:getAnimationTime(self.coverAnimation), true)
        else
            self:playAnimation(self.coverAnimation, 1, self:getAnimationTime(self.coverAnimation), true)
        end
    end
end

function LivestockTrailerExtension:setDoorState(doorsOpen, noEventSend)
	if self.doorsAnimation ~= nil then
		SetDoorStateEvent.sendEvent(self, doorsOpen, noEventSend)
		self.doorsOpen = doorsOpen

		local animationTime = self:getAnimationTime(self.doorsAnimation)
		if doorsOpen and animationTime < 1 then
			self:playAnimation(self.doorsAnimation, 1, animationTime, true)
		elseif doorsOpen and animationTime >= 1 then
			self:playAnimation(self.doorsAnimation, 1, 0, true)
		elseif animationTime > 0 then
			self:playAnimation(self.doorsAnimation, -1, animationTime, true)
		end

	end
end

function LivestockTrailerExtension:isInTriggerNode()
	if self.nodeIndex ~= nil then
		local wx, wy, wz = getWorldTranslation(g_currentMission.player.rootNode)
		local tx, ty, tz = getWorldTranslation(self.nodeIndex)
		local distance = Utils.vector3Length(wx-tx, wy-ty, wz-tz)

		if distance <= self.triggerDistance then
			return true
		else
			return false
		end
	end
end

function LivestockTrailerExtension:animalsLoaded()
	if self.getFillLevel ~= nil then
		return self:getFillLevel() > 0
	end
	return false
end

function LivestockTrailerExtension:isUsedOn()
	local rootAttacherVehicle = self:getRootAttacherVehicle()
	if rootAttacherVehicle ~= nil and rootAttacherVehicle.motor ~= nil then
		return true
	end
	return false
end

function LivestockTrailerExtension:mouseEvent(posX, posY, isDown, isUp, button)
end

function LivestockTrailerExtension:keyEvent(unicode, sym, modifier, isDown)
end

function LivestockTrailerExtension:delete()
end

-- Doors Event
SetDoorStateEvent = {}
SetDoorStateEvent_mt = Class(SetDoorStateEvent, Event)
InitEventClass(SetDoorStateEvent, "SetDoorStateEvent")

function SetDoorStateEvent:emptyNew()
    local self = Event:new(SetDoorStateEvent_mt)
    return self
end

function SetDoorStateEvent:new(object, doorsOpen)
    local self = SetDoorStateEvent:emptyNew()
    self.object = object
    self.doorsOpen = doorsOpen
    return self
end

function SetDoorStateEvent:readStream(streamId, connection)
    self.object = readNetworkNodeObject(streamId)
    self.doorsOpen = streamReadBool(streamId)
    self:run(connection)
end

function SetDoorStateEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object)
    streamWriteBool(streamId, self.doorsOpen)
end

function SetDoorStateEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.object)
    end
    if self.object ~= nil then
        self.object:setDoorState(self.doorsOpen, true)
    end
end

function SetDoorStateEvent.sendEvent(vehicle, doorsOpen, noEventSend)
    if vehicle.doorsOpen ~= doorsOpen then
        if noEventSend == nil or noEventSend == false then
            if g_server ~= nil then
                g_server:broadcastEvent(SetDoorStateEvent:new(vehicle, doorsOpen), nil, nil, vehicle)
            else
                g_client:getServerConnection():sendEvent(SetDoorStateEvent:new(vehicle, doorsOpen))
            end
        end
    end
end